//
//  ColorPalete.swift
//  R.swift
//
//  Created by Tom Lokhorst on 2016-03-13.
//  Copyright © 2016 Mathijs Kadijk. All rights reserved.
//

import Foundation
import AppKit.NSColor

struct ColorPalette: WhiteListedExtensionsResourceType {
  static let supportedExtensions: Set<String> = ["clr"]

  let filename: String
  let colors: [String: NSColor]

  init(url: NSURL) throws {
    try ColorPalette.throwIfUnsupportedExtension(url.pathExtension)

    guard let filename = url.filename, path = url.path else {
      throw ResourceParsingError.ParsingFailed("Couldn't extract filename without extension from URL: \(url)")
    }
    guard let colorList = NSColorList(name: "", fromFile: path) else {
      throw ResourceParsingError.ParsingFailed("Couldn't parse as color palette: \(url)")
    }

    var colors: [String: NSColor] = [:]
    for key in colorList.allKeys {
      colors[key] = colorList.colorWithKey(key)?.colorUsingColorSpaceName(NSCalibratedRGBColorSpace)
    }

    self.filename = filename
    self.colors = colors
  }
}
